package consume

import (
	"applet/app/db"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	esMd "applet/es/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/es"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/cc14514/go-geoip2"
	geoip2db "github.com/cc14514/go-geoip2-db"
	"github.com/streadway/amqp"
	"net"
	"strings"
	"time"
	"xorm.io/xorm"
)

func CanalGuideOrderConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1000)
	delivery := ch.Consume(queue.Name, true) //设置自动应答

	geoIp2db, _ := geoip2db.NewGeoipDbByStatik()
	defer geoIp2db.Close()

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>>>CanalGuideOrderConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleGuideOrdTable(res.Body, geoIp2db)
			//_ = res.Reject(false)
			//_ = res.Ack(true)
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleGuideOrdTable(msg []byte, geoIp2db *geoip2.DBReader) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalOrderMessage[md.CanalGuideOrder]
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return err
	}

	//2、判断操作(目前只针对insert进行修改)
	if canalMsg.Type == md.CanalMsgInsertSqlType {
		//3、TODO::判断es索引是否创建(因为这里我已经手动创建了,省略此步骤)
		//err := es.CreateIndexIfNotExists(esMd.ZhiosOrdersEsIndex, esMd.ZhiosOrdersEsMapping)
		//if err != nil {
		//	panic(err)
		//}

		//4、构造数据结构体 && 插入es文档
		masterId := strings.Split(canalMsg.Database, "_")[1]
		for _, data := range canalMsg.Data {
			var uniqueId = masterId + "_" + data.OrdId //es的唯一键，每个文档必须要有唯一键
			user, err := db.UserFindByID(db.DBs[masterId], data.Uid)
			if err != nil {
				return err
			}
			provinceId, cityId, countyId, provinceName, cityName, countyName := getUserAddress(db.DBs[masterId], data.Uid, geoIp2db)
			now := time.Now()
			esData := esMd.ZhiosOrdersEs{
				OrdKind:    "guide",
				MasterId:   utils.StrToInt(masterId),
				OrdId:      data.OrdId,
				MainOrdId:  data.ParentOrdId,
				StoreOrdId: data.PvdOid,
				GoodsId:    data.ItemId,
				Uid:        data.Uid,
				BuyerPhone: user.Phone,
				CostPrice:  utils.StrToFloat64(data.PaidPrice),
				//Gmv:                   utils.StrToFloat64(data.CostPrice) * float64(utils.StrToInt(data.ItemNum)),
				Gmv:                   utils.StrToFloat64(data.PaidPrice),
				State:                 int32(utils.StrToInt(data.State)),
				StateZh:               md.CanalGuideOrderState(int32(utils.StrToInt(data.State))).String(),
				GoodsNum:              utils.StrToInt(data.ItemNum),
				PayChannel:            0,
				PayChannelZh:          "",
				ReceiverPhone:         user.Phone,
				ReceiverName:          user.Nickname,
				ReceiverAddressDetail: "",
				ProvinceId:            provinceId,
				CityId:                cityId,
				CountyId:              countyId,
				Province:              provinceName,
				City:                  cityName,
				County:                countyName,
				OrderType:             int32(utils.StrToInt(data.OrderType)),
				EstimateCommission:    utils.StrToFloat64(data.SysCommission),
				LogisticCompany:       "",
				PayTime:               "",
				OrdCreateTime:         utils.TimeToStr(data.CreateAt),
				CreatedAt:             now.Format("2006-01-02 15:04:05"),
				UpdatedAt:             now.Format("2006-01-02 15:04:05"),
			}
			if esData.GoodsNum == 0 {
				esData.GoodsNum = 1
			}
			if esData.PayTime == "" {
				esData.PayTime = "1970-01-01 00:00:00"
			}
			createDocRet, err := es.CreateDoc(esMd.ZhiosOrdersEsIndex, uniqueId, esData)
			if err != nil {
				return err
			}
			fmt.Printf("CreateDoc CanalGuide ==> %+v \n\n", createDocRet)
		}
	}
	return nil
}

func getUserAddress(Db *xorm.Engine, uid string, geoIp2db *geoip2.DBReader) (provinceId, cityId, countyId, provinceName, cityName, countyName string) {
	regionalAgentUserBelong, err := db.RegionalAgentUserBelongGetOneByParams(Db, map[string]interface{}{
		"key":   "uid",
		"value": uid,
	})
	if err != nil {
		return
	}
	if regionalAgentUserBelong == nil {
		//TODO::没开区域代理的,使用用户ip归属地
		userLoginIpList, err := db.UserLoginIpListGetOneByParams(Db, map[string]interface{}{
			"key":   "uid",
			"value": uid,
		})
		if err != nil || userLoginIpList == nil {
			provinceName = "广东省"
			provinceId = "440000000000"
			return
		}

		utils.FilePutContents("getUserAddress", utils.SerializeStr(userLoginIpList))
		record, _ := geoIp2db.City(net.ParseIP(userLoginIpList.NowIp))
		if record.Country.Names != nil && record.Subdivisions != nil && record.City.Names != nil {
			province, err := db.ProvinceGetOneByName(record.Subdivisions[0].Names["zh-CN"])
			if err != nil {
				return
			}
			provinceId = province.Id
			provinceName = province.Name

			city, err := db.CityGetOneByName(record.City.Names["zh-CN"])
			if err != nil {
				return
			}
			cityId = city.Id
			cityName = city.Name
			return
		}
		//TODO::没开区域代理的，默认归属到广东
		provinceName = "广东省"
		provinceId = "440000000000"
		return
	}
	var regionalId int64
	if regionalAgentUserBelong.DistrictId != 0 {
		regionalId = regionalAgentUserBelong.DistrictId
	} else {
		regionalId = int64(regionalAgentUserBelong.RegionId)
	}
	regionalAgentRegion, err := db.RegionalAgentRegionGetOneByParams(Db, map[string]interface{}{
		"key":   "id",
		"value": regionalId,
	})
	if err != nil || regionalAgentRegion.WebsiteRegionId == "" {
		return "", "", "", "", "", ""
	}
	switch regionalAgentRegion.Level {
	case 1:
		province, err := db.ProvinceGetOne(regionalAgentRegion.WebsiteRegionId)
		if err != nil {
			return "", "", "", "", "", ""
		}
		provinceId = province.Id
		provinceName = province.Name
		break
	case 2:
		city, err := db.CityGetOne(regionalAgentRegion.WebsiteRegionId)
		if err != nil {
			return "", "", "", "", "", ""
		}
		province, err := db.ProvinceGetOne(city.ProvinceId)
		if err != nil {
			return "", "", "", "", "", ""
		}
		provinceId = province.Id
		provinceName = province.Name
		cityId = city.Id
		cityName = city.Name
		break
	case 3:
		county, err := db.CountyGetOne(regionalAgentRegion.WebsiteRegionId)
		if err != nil {
			return "", "", "", "", "", ""
		}
		city, err := db.CityGetOne(county.CityId)
		if err != nil {
			return "", "", "", "", "", ""
		}
		province, err := db.ProvinceGetOne(city.ProvinceId)
		if err != nil {
			return "", "", "", "", "", ""
		}
		provinceId = province.Id
		provinceName = province.Name
		cityId = city.Id
		cityName = city.Name
		countyId = county.Id
		countyName = county.Name
		break
	}
	return
}
