package consume

import (
	"applet/app/db"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	esMd "applet/es/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/es"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"strings"
	"time"
)

func CanalOrderConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>CanalOrderConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleMallOrdTable(res.Body)
			//_ = res.Reject(false)
			_ = res.Ack(true)
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleMallOrdTable(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalOrderMessage[md.CanalOrder]
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return err
	}
	//2、判断操作(目前只针对insert进行修改)
	if canalMsg.Type == md.CanalMsgInsertSqlType {
		//3、TODO::判断es索引是否创建(因为这里我已经手动创建了,省略此步骤)
		//err := es.CreateIndexIfNotExists(esMd.ZhiosOrdersEsIndex, esMd.ZhiosOrdersEsMapping)
		//if err != nil {
		//	panic(err)
		//}

		//4、构造数据结构体 && 插入es文档
		masterId := strings.Split(canalMsg.Database, "_")[1]

		var ordList []string
		for _, data := range canalMsg.Data {
			ordList = append(ordList, data.OrdId)
		}
		ordItemList, err := db.MallOrdItemFindByParams(db.DBs[masterId], map[string]interface{}{
			"key":   "ord_id",
			"value": ordList,
		})
		if err != nil {
			return err
		}
		var ordItemListMap = map[string]int{}
		for _, item := range *ordItemList {
			ordItemListMap[utils.AnyToString(item.OrdId)] = item.GoodsNum
		}

		for _, data := range canalMsg.Data {
			var uniqueId = masterId + "_" + data.OrdId //es的唯一键，每个文档必须要有唯一键
			province, err := db.ProvinceGetOne(data.ProvinceId)
			if err != nil {
				return err
			}
			city, err := db.CityGetOne(data.CityId)
			if err != nil {
				return err
			}
			county, err := db.CountyGetOne(data.CountyId)
			if err != nil {
				return err
			}
			now := time.Now()
			var goodsNum = 1
			if ordItemListMap[data.OrdId] != 0 {
				goodsNum = ordItemListMap[data.OrdId]
			}
			esData := esMd.ZhiosOrdersEs{
				OrdKind:               "mall",
				MasterId:              utils.StrToInt(masterId),
				OrdId:                 data.OrdId,
				MainOrdId:             data.MainOrdId,
				StoreOrdId:            data.StoreOrdId,
				GoodsId:               data.GoodsId,
				Uid:                   data.Uid,
				BuyerPhone:            data.BuyerPhone,
				CostPrice:             utils.StrToFloat64(data.CostPrice),
				Gmv:                   utils.StrToFloat64(data.CostPrice) * float64(goodsNum),
				State:                 int32(utils.StrToInt(data.State)),
				StateZh:               md.CanalOrderState(int32(utils.StrToInt(data.State))).String(),
				PayChannel:            int32(utils.StrToInt(data.PayChannel)),
				PayChannelZh:          md.CanalOrderPayChannel(int32(utils.StrToInt(data.PayChannel))).String(),
				ReceiverPhone:         data.ReceiverPhone,
				ReceiverName:          data.ReceiverName,
				ReceiverAddressDetail: data.ReceiverAddressDetail,
				ProvinceId:            data.ProvinceId,
				CityId:                data.CityId,
				CountyId:              data.CountyId,
				Province:              province.Name,
				City:                  city.Name,
				County:                county.Name,
				OrderType:             int32(utils.StrToInt(data.OrderType)),
				GoodsNum:              goodsNum,
				EstimateCommission:    utils.StrToFloat64(data.EstimateCommission),
				LogisticCompany:       data.LogisticCompany,
				PayTime:               data.PayTime,
				OrdCreateTime:         data.CreateTime,
				CreatedAt:             now.Format("2006-01-02 15:04:05"),
				UpdatedAt:             now.Format("2006-01-02 15:04:05"),
			}
			if esData.PayTime == "" {
				esData.PayTime = "1970-01-01 00:00:00"
			}
			createDocRet, err := es.CreateDoc(esMd.ZhiosOrdersEsIndex, uniqueId, esData)
			if err != nil {
				return err
			}
			fmt.Printf("CreateDoc CanalOrder ==> %+v \n\n", createDocRet)
		}
	}
	return nil
}
