package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	"applet/app/utils"
	"applet/app/utils/cache"
	tool "applet/app/utils/cloud_issuance"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"strconv"
	"time"
)

func CloudIssuanceAsyncMLoginConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>CloudIssuanceAsyncMLoginConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			//解析mq中queue的数据结构体
			var msg *md.CloudIssuanceAsyncMLogin
			err = json.Unmarshal(res.Body, &msg)
			if err != nil {
				panic(err)
			}
			go func() {
				err := handleAsyncMLogin(msg.WId, msg.MasterId, msg.UserId, msg.RobotId)
				if err != nil {
					fmt.Println("<<<<<<<<<<<< handleAsyncMLogin err>>>>>>>>", err.Error())
				}
			}()
			_ = res.Ack(true)
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleAsyncMLogin(wId, masterId, userId string, robotId int) error {
	var timeTotal = 0
	for {
		err, resp := robotAsyncMacLogin(md.RobotAsyncMacLoginRequest{
			WId:     wId,
			RobotId: robotId,
		})
		if err == nil {
			fmt.Println("success <<<<<<<", resp)
			if cfg.Prd {
				engine := db.DBs[masterId]
				cloudIssuanceUserRobotDb := db.CloudIssuanceUserRobotDb{}
				cloudIssuanceUserRobotDb.Set(engine, utils.StrToInt(userId))
				robot, err := cloudIssuanceUserRobotDb.GetCloudIssuanceUserRobot()
				if err != nil {
					return err
				}
				if robot == nil {
					return errors.New("未查询到机器人记录")
				}
				robot.WcId = resp.WcId
				robot.WcNickname = resp.NickName
				robot.WcHeadUrl = resp.HeadUrl
				robot.LoginStatus = 1
				_, err = cloudIssuanceUserRobotDb.SaveCloudIssuanceUserRobot(robot.Id, robot, "wc_id", "wc_nickname", "wc_head_url", "login_status")
				if err != nil {
					return err
				}
			} else {
				//TODO::测试环境, 简化操作 通过redis 桥接
				cache.SetEx(fmt.Sprintf("cloud_issuance_async_login_success:%s:%s", masterId, userId), utils.SerializeStr(resp), 60*5)
			}
			return nil
		}

		fmt.Println("err>>>>>>>>", err.Error())
		if err != nil && err.Error() != "未登录" {
			return err
		}
		//TODO::休眠5s
		time.Sleep(time.Duration(5) * time.Second)
		timeTotal += 5
		if timeTotal >= 240 {
			return errors.New("超时未登录")
		}
	}
}

func robotAsyncMacLogin(args md.RobotAsyncMacLoginRequest) (err error, result md.RobotAsyncMacLoginResponse) { // 同步登录状态
	url := tool.HttpBuild("itaoke.robot.async.mlogin", "127.0.0.1", map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
		"wId":      args.WId,
	})
	data, err := tool.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	err = json.Unmarshal(resByte, &result)
	if err != nil {
		return
	}
	return
}
